import { App, Config, Inject, InjectClient, Provide } from '@midwayjs/core';
import { Application, Context } from '@midwayjs/koa';
import { RequestBase } from '../interface/proxy.interface';
import { get, lowerCase, omit, pick } from 'lodash';
import { HttpServiceFactory, HttpService } from '@midwayjs/axios';
import { PemService } from './pem.service';
import { ErrorCode, ServiceError } from '../error/service.error';
@Provide()
export class ProxyService {
  @Inject()
  ctx: Context;
  @App()
  app: Application;

  @Config('axios.clients')
  axiosClients: object;

  @Config('useCrypto')
  useCrypto: boolean;

  @InjectClient(HttpServiceFactory, 'default')
  serviceAxios: HttpService;

  @Inject()
  sf: HttpServiceFactory;

  @Inject()
  pemService: PemService;
  /**
   * 组织请求所需要的参数
   * @returns {RequestBase} result
   */
  getRequstBase(): RequestBase {
    const request = this.ctx.request;
    const result: RequestBase = pick(request, ['method', 'header', 'path', 'query', 'querystring', 'body', 'rawBody', 'host', 'ip', 'originalUrl']) as RequestBase;
    // 加密后的长度与解密后的长度不一致,会导致服务无法解析
    delete result.header['content-length'];
    if (this.useCrypto) {
      if (result.body && result.rawBody) {
        result.body = JSON.parse(result.rawBody);
      }
    }
    return result;
  }
  /**
   * 代理执行函数
   * @returns
   */
  async index() {
    const rb: RequestBase = this.getRequstBase();
    // 解析路由前缀
    const uri = rb.originalUrl;
    const keys = Object.keys(this.axiosClients);
    let url;
    for (const prefix of keys) {
      const regRules = `^(${prefix})`;
      const reg = new RegExp(regRules);
      const res = reg.test(uri);
      if (!res) continue;
      const clientConfig = this.axiosClients[prefix];
      const baseURL = clientConfig.baseURL;
      // 去掉前缀
      // url = uri.replace(reg, baseURL);
      // 保留前缀(目前采用这一方式处理)
      url = `${baseURL}${rb.originalUrl}`;
      break;
    }
    if (!url) return;
    let reqConfig: any = {
      url,
      method: rb.method,
      headers: rb.header,
    };
    if (lowerCase(rb.method) === 'post') {
      // 如果是post函数,且 不属于开发模式,则解密函数体.否则不用解密
      if (this.useCrypto) {
        const preBody = get(rb.body, 'data');
        const deBody = this.pemService.decrypt(preBody);
        if (deBody) {
          reqConfig.data = JSON.parse(deBody);
          delete reqConfig.rawBody;
        }
      } else {
        reqConfig.data = rb.body;
      }
    }
    reqConfig = JSON.parse(JSON.stringify(reqConfig));
    let result;
    try {
      const res = await this.serviceAxios.request(reqConfig);
      if (res.status !== 200) throw new ServiceError(ErrorCode.REQUSET_ERROR);
      result = res.data;
    } catch (error) {
      // 请求不是自定义的错误,把错误隐藏,写到日志中
      this.ctx.logger.error(error.stack);
      console.log(error);
      throw new ServiceError(ErrorCode.REQUSET_ERROR);
      // return {
      //   errmsg: 'proxy service request error',
      //   errcode: get(error, 'response.status'),
      // };
    }
    if (result.errcode === 0) {
      if (this.useCrypto) {
        // 需要将返回的数据加密成字符串
        if (get(result, 'data')) {
          const enReturn = this.pemService.encrypt(omit(result, ['errcode', 'errmsg']));
          result.data = enReturn;
        }
      }
      return result;
    } else {
      // 需要将异常信息翻译并返回
      console.error(result);
      throw new ServiceError(result.errcode);
    }
  }
}
