import { Config, Inject, Provide } from '@midwayjs/core';
import { JwtService } from '@midwayjs/jwt';
import { Context } from '@midwayjs/koa';
import { get, toLower } from 'lodash';
import { ServiceError, ErrorCode } from '../error/service.error';
import * as Crypto from 'crypto-js';
import { RedisService } from '@midwayjs/redis';
import { ProxyService } from './proxy.service';

@Provide()
export class SingleSignOnService {
  @Inject()
  private ctx: Context;
  @Config('loginSign')
  private loginSign;
  @Config('jwt.secret')
  private jwtSecret;
  @Config('jwt.expiresIn')
  private jwtExpiresIn;
  /**核心服务前缀 */
  @Config('projectPrefix.core')
  corePrefix: string;
  @Inject()
  proxyService: ProxyService;

  @Inject()
  private jwtService: JwtService;
  @Inject()
  private redisService: RedisService;
  /**
   * 单点登录检查
   */
  async index(): Promise<void> {
    let user;
    const token: any = get(this.ctx.request, 'header.token');
    if (token) {
      const data = this.jwtService.decodeSync(token);
      if (data) user = data;
    }
    if (!user) throw new ServiceError(ErrorCode.NOT_LOGIN);
    const { id, role, login_code } = user;
    if (!login_code) throw new ServiceError(ErrorCode.NOT_LOGIN);
    // 解密
    const decodeResult = Crypto.AES.decrypt(login_code, this.jwtSecret);
    const decode = Crypto.enc.Utf8.stringify(decodeResult).toString();
    // 取出code
    const codeArr = decode.split(':');
    const code = codeArr[codeArr.length - 1];
    // 拼接redis的key
    const rediskey = `${this.loginSign}:${role}:${id}`;
    // 取出当前记录在案的 code
    const redisCode = await this.redisService.get(rediskey);
    if (!redisCode) throw new ServiceError(ErrorCode.ACCOUNT_HAS_EXPIRED);
    // 判断是否一致
    if (code === redisCode) {
      // 一致,延时
      await this.redisService.expire(rediskey, this.jwtExpiresIn);
    } else {
      throw new ServiceError(ErrorCode.ACCOUNT_LOGGED_IN_ELESWHERE);
    }
  }
  /**
   * 检查路由是否在白名单中
   */
  inWhiteList(): boolean {
    const whiteList = [
      {
        uri: `${this.corePrefix}/login`,
        method: 'post',
        desc: '登陆接口',
        type: 'includes',
      },
      {
        uri: `${this.corePrefix}/dict`,
        method: 'get',
        desc: '字典接口',
      },
      {
        uri: `${this.corePrefix}/token/tokenView`,
        method: 'get',
        desc: 'token认证',
      },
    ];
    const rb = this.proxyService.getRequstBase();
    const r = whiteList.find(f => {
      let pathRes = false;
      const methodRes = toLower(f.method) === toLower(rb.method);
      if (f.type && f.type === 'includes') {
        pathRes = rb.path.includes(f.uri);
      } else {
        pathRes = f.uri === rb.path;
      }
      return pathRes && methodRes;
    });
    // return r ? true : false;
    return true;
  }
}
